@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache.h 5046 2010-07-13 10:26:47Z tfheen $
+ * $Id: cache.h 5312 2010-09-28 12:28:59Z tfheen $
  */
 
 /*
@@ -376,6 +376,9 @@ struct sess {
 	int			esis;
 	int			disable_esi;
 
+	uint8_t			hash_ignore_busy;
+	uint8_t			hash_always_miss;
+
 	struct worker		*wrk;
 
 	socklen_t		sockaddrlen;
@@ -387,6 +390,7 @@ struct sess {
 	/* formatted ascii client address */
 	char			*addr;
 	char			*port;
+	char			*client_identity;
 
 	/* HTTP request */
 	const char		*doclose;
@@ -413,6 +417,7 @@ struct sess {
 	enum step		step;
 	unsigned		cur_method;
 	unsigned		handling;
+	unsigned char		pass;
 	unsigned char		sendbody;
 	unsigned char		wantbody;
 	int			err_code;
@@ -450,6 +455,8 @@ struct vbe_conn {
 	struct backend		*backend;
 	int			fd;
 
+	uint8_t			recycled;
+
 	/* Timeouts */
 	double			first_byte_timeout;
 	double			between_bytes_timeout;
@@ -471,7 +478,7 @@ extern pthread_t VCA_thread;
 struct vbe_conn *VBE_GetFd(const struct director *, struct sess *sp);
 int VBE_Healthy(double now, const struct director *, uintptr_t target);
 int VBE_Healthy_sp(const struct sess *sp, const struct director *);
-void VBE_ClosedFd(struct sess *sp);
+void VBE_CloseFd(struct sess *sp);
 void VBE_RecycleFd(struct sess *sp);
 void VBE_AddHostHeader(const struct sess *sp);
 void VBE_Poll(void);
