@@ -33,7 +33,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_backend.c 4630 2010-03-18 10:13:34Z phk $")
+SVNID("$Id: cache_backend.c 5434 2010-10-19 15:01:04Z tfheen $")
 
 #include <limits.h>
 #include <stdio.h>
@@ -78,6 +78,7 @@ VBE_ReleaseConn(struct vbe_conn *vc)
 	assert(vc->backend == NULL);
 	assert(vc->fd < 0);
 
+	vc->recycled = 0;
 	if (params->cache_vbe_conns) {
 		Lck_Lock(&VBE_mtx);
 		VTAILQ_INSERT_HEAD(&vbe_conns, vc, list);
@@ -339,11 +340,20 @@ vbe_GetVbe(struct sess *sp, struct backend *bp)
 			VSL_stats->backend_reuse += 1;
 			WSP(sp, SLT_Backend, "%d %s %s",
 			    vc->fd, sp->director->vcl_name, bp->vcl_name);
+			vc->recycled = 1;
 			return (vc);
 		}
 		VSL_stats->backend_toolate++;
-		sp->vbe = vc;
-		VBE_ClosedFd(sp);
+		WSL(sp->wrk, SLT_BackendClose, vc->fd, "%s", bp->vcl_name);
+
+		/* Checkpoint log to flush all info related to this connection
+		   before the OS reuses the FD */
+		WSL_Flush(sp->wrk, 0);
+
+		TCP_close(&vc->fd);
+		VBE_DropRefConn(bp);
+		vc->backend = NULL;
+		VBE_ReleaseConn(vc);
 	}
 
 	if (!vbe_Healthy(sp->t_req, (uintptr_t)sp->objhead, bp)) {
@@ -375,7 +385,7 @@ vbe_GetVbe(struct sess *sp, struct backend *bp)
 /* Close a connection ------------------------------------------------*/
 
 void
-VBE_ClosedFd(struct sess *sp)
+VBE_CloseFd(struct sess *sp)
 {
 	struct backend *bp;
 
@@ -386,6 +396,11 @@ VBE_ClosedFd(struct sess *sp)
 	bp = sp->vbe->backend;
 
 	WSL(sp->wrk, SLT_BackendClose, sp->vbe->fd, "%s", bp->vcl_name);
+
+	/* Checkpoint log to flush all info related to this connection
+	   before the OS reuses the FD */
+	WSL_Flush(sp->wrk, 0);
+
 	TCP_close(&sp->vbe->fd);
 	VBE_DropRefConn(bp);
 	sp->vbe->backend = NULL;
@@ -469,6 +484,24 @@ struct vdi_simple {
 	struct backend		*backend;
 };
 
+/* Returns the backend if and only if the this is a simple director.
+ * XXX: Needs a better name and possibly needs a better general approach.
+ * XXX: This is mainly used by the DNS director to fetch the actual backend
+ * XXX: so it can compare DNS lookups with the actual IP.
+ */
+struct backend *
+vdi_get_backend_if_simple(const struct director *d)
+{
+	CHECK_OBJ_NOTNULL(d, DIRECTOR_MAGIC);
+	struct vdi_simple *vs, *vs2;
+
+	vs2 = d->priv;
+	if (vs2->magic != VDI_SIMPLE_MAGIC)
+		return NULL;
+	CAST_OBJ_NOTNULL(vs, d->priv, VDI_SIMPLE_MAGIC);
+	return vs->backend;
+}
+
 static struct vbe_conn *
 vdi_simple_getfd(const struct director *d, struct sess *sp)
 {
