@@ -33,7 +33,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_backend_cfg.c 5053 2010-07-28 11:11:56Z tfheen $")
+SVNID("$Id: cache_backend_cfg.c 5442 2010-10-20 13:37:02Z tfheen $")
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -273,6 +273,8 @@ VRT_init_dir(struct cli *cli, struct director **dir, const char *name,
 		VRT_init_dir_hash(cli, dir, idx, priv);
 	else if (!strcmp(name, "random"))
 		VRT_init_dir_random(cli, dir, idx, priv);
+	else if (!strcmp(name, "dns"))
+		VRT_init_dir_dns(cli, dir, idx, priv);
 	else if (!strcmp(name, "round-robin"))
 		VRT_init_dir_round_robin(cli, dir, idx, priv);
 	else if (!strcmp(name, "client"))
