@@ -58,7 +58,7 @@ DOT acceptor -> start [style=bold,color=green,weight=4]
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_center.c 5050 2010-07-28 09:54:29Z tfheen $")
+SVNID("$Id: cache_center.c 5317 2010-09-28 13:25:59Z tfheen $")
 
 #include <stdio.h>
 #include <errno.h>
@@ -269,6 +269,8 @@ cnt_done(struct sess *sp)
 	memset(&sp->acct_req, 0, sizeof sp->acct_req);
 
 	sp->t_req = NAN;
+	sp->hash_always_miss = 0;
+	sp->hash_ignore_busy = 0;
 
 	if (sp->fd >= 0 && sp->doclose != NULL) {
 		/*
@@ -400,7 +402,7 @@ DOT		label="fetch from backend\n(find obj.ttl)"
 DOT	]
 DOT	vcl_fetch [
 DOT		shape=record
-DOT		label="vcl_fetch()|req.\nobj.\nbereq.\nberesp."
+DOT		label="vcl_fetch()|req.\nbereq.\nberesp."
 DOT	]
 DOT	fetch -> vcl_fetch [style=bold,color=blue,weight=2]
 DOT	fetch_pass [
@@ -440,6 +442,15 @@ cnt_fetch(struct sess *sp)
 	http_Setup(sp->wrk->beresp, sp->wrk->ws);
 
 	i = FetchHdr(sp);
+	/*
+	 * If we recycle a backend connection, there is a finite chance
+	 * that the backend closed it before we get a request to it.
+	 * Do a single retry in that case.
+	 */
+	if (i == 1) {
+		VSL_stats->backend_retry++;
+		i = FetchHdr(sp);
+	}
 
 	/*
 	 * Save a copy before it might get mangled in VCL.  When it comes to
@@ -533,7 +544,7 @@ cnt_fetch(struct sess *sp)
 		AZ(sp->objcore);
 	}
 
-	l = http_EstimateWS(sp->wrk->beresp, HTTPH_A_INS, &nhttp);
+	l = http_EstimateWS(sp->wrk->beresp, sp->pass ? HTTPH_R_PASS : HTTPH_A_INS, &nhttp);
 
 	if (vary != NULL)
 		l += varyl;
@@ -585,7 +596,7 @@ cnt_fetch(struct sess *sp)
 
 	hp2->logtag = HTTP_Obj;
 	http_CopyResp(hp2, hp);
-	http_FilterFields(sp->wrk, sp->fd, hp2, hp, HTTPH_A_INS);
+	http_FilterFields(sp->wrk, sp->fd, hp2, hp, sp->pass ? HTTPH_R_PASS : HTTPH_A_INS);
 	http_CopyHome(sp->wrk, sp->fd, hp2);
 
 	if (http_GetHdr(hp, H_Last_Modified, &b))
@@ -950,6 +961,7 @@ cnt_pass(struct sess *sp)
 	sp->acct_tmp.pass++;
 	sp->sendbody = 1;
 	sp->step = STP_FETCH;
+	sp->pass = 1;
 	return (0);
 }
 
@@ -1037,6 +1049,8 @@ cnt_recv(struct sess *sp)
 	AN(sp->director);
 
 	sp->disable_esi = 0;
+	sp->pass = 0;
+	sp->client_identity = NULL;
 
 	VCL_recv_method(sp);
 	recv_handling = sp->handling;
