@@ -37,7 +37,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_cli.c 4557 2010-02-12 11:58:43Z phk $")
+SVNID("$Id: cache_cli.c 5325 2010-09-29 07:38:21Z tfheen $")
 
 #include <stdio.h>
 #include <errno.h>
@@ -86,6 +86,7 @@ static void
 cli_cb_before(const struct cli *cli)
 {
 
+	ASSERT_CLI();
 	VSL(SLT_CLI, 0, "Rd %s", cli->cmd);
 	VCL_Poll();
 	VBE_Poll();
@@ -95,8 +96,11 @@ cli_cb_before(const struct cli *cli)
 static void
 cli_cb_after(const struct cli *cli)
 {
+
+	ASSERT_CLI();
 	Lck_Unlock(&cli_mtx);
-	VSL(SLT_CLI, 0, "Wr %03u %s", cli->result, vsb_data(cli->sb));
+	VSL(SLT_CLI, 0, "Wr %03u %u %s",
+	    cli->result, vsb_len(cli->sb), vsb_data(cli->sb));
 }
 
 void
@@ -167,7 +171,6 @@ static struct cli_proto master_cmds[] = {
 	{ NULL }
 };
 
-
 /*--------------------------------------------------------------------
  * Initialize the CLI subsystem
  */
@@ -184,4 +187,3 @@ CLI_Init(void)
 
 	CLI_AddFuncs(master_cmds);
 }
-
