@@ -42,7 +42,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_dir_random.c 4557 2010-02-12 11:58:43Z phk $")
+SVNID("$Id: cache_dir_random.c 5303 2010-09-28 09:28:24Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -106,7 +106,11 @@ vdi_random_getfd(const struct director *d, struct sess *sp)
 		 */
 		SHA256_Init(&ctx);
 		AN(sp->addr);
-		SHA256_Update(&ctx, sp->addr, strlen(sp->addr));
+		if (sp->client_identity != NULL)
+			SHA256_Update(&ctx, sp->client_identity,
+			    strlen(sp->client_identity));
+		else
+			SHA256_Update(&ctx, sp->addr, strlen(sp->addr));
 		SHA256_Final(sign, &ctx);
 		hp = sign;
 	}
