@@ -30,7 +30,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_fetch.c 5044 2010-07-13 09:38:04Z tfheen $")
+SVNID("$Id: cache_fetch.c 5364 2010-09-30 08:25:24Z tfheen $")
 
 #include <stdio.h>
 #include <inttypes.h>
@@ -211,7 +211,7 @@ fetch_chunked(struct sess *sp, struct http_conn *htc)
 	if (st != NULL && st->len == 0) {
 		VTAILQ_REMOVE(&sp->obj->store, st, list);
 		STV_free(st);
-	} else if (st != NULL)
+	} else if (st != NULL && st->len < st->space)
 		STV_trim(st, st->len);
 	return (0);
 }
@@ -276,7 +276,7 @@ fetch_eof(struct sess *sp, struct http_conn *htc)
 	if (st->len == 0) {
 		VTAILQ_REMOVE(&sp->obj->store, st, list);
 		STV_free(st);
-	} else
+	} else 
 		STV_trim(st, st->len);
 
 	return (1);
@@ -325,7 +325,15 @@ FetchReqBody(struct sess *sp)
 	return (0);
 }
 
-/*--------------------------------------------------------------------*/
+/*--------------------------------------------------------------------
+ * Send request, and receive the HTTP protocol response, but not the
+ * response body.
+ *
+ * Return value:
+ *	-1 failure, not retryable
+ *	 0 success
+ *	 1 failure which can be retried.
+ */
 
 int
 FetchHdr(struct sess *sp)
@@ -334,35 +342,38 @@ FetchHdr(struct sess *sp)
 	struct worker *w;
 	char *b;
 	struct http *hp;
+	int retry = -1;
 	int i;
-	double tmo;
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->wrk, WORKER_MAGIC);
+	w = sp->wrk;
+
 	AN(sp->director);
 	AZ(sp->obj);
+
 	if (sp->objcore != NULL) {		/* pass has no objcore */
 		CHECK_OBJ_NOTNULL(sp->objcore, OBJCORE_MAGIC);
 		AN(sp->objhead);		/* details in hash_slinger.h */
 		AN(sp->objcore->flags & OC_F_BUSY);
 	}
 
-	/* Transmit request */
-
-	w = sp->wrk;
 	hp = sp->wrk->bereq;
 
 	sp->vbe = VBE_GetFd(NULL, sp);
 	if (sp->vbe == NULL) {
 		WSP(sp, SLT_FetchError, "no backend connection");
-		return (__LINE__);
+		return (-1);
 	}
 	vc = sp->vbe;
+	if (vc->recycled)
+		retry = 1;
+		
 
 	/*
 	 * Now that we know our backend, we can set a default Host:
-	 * header if one is necessary.
-	 * XXX: This possibly ought to go into the default VCL
+	 * header if one is necessary.  This cannot be done in the VCL
+	 * because the backend may be chosen by a director.
 	 */
 	if (!http_GetHdr(hp, H_Host, &b))
 		VBE_AddHostHeader(sp);
@@ -374,10 +385,11 @@ FetchHdr(struct sess *sp)
 	/* Deal with any message-body the request might have */
 	i = FetchReqBody(sp);
 	if (WRW_FlushRelease(w) || i > 0) {
-		WSP(sp, SLT_FetchError, "backend write error: %d", errno);
-		VBE_ClosedFd(sp);
+		WSP(sp, SLT_FetchError, "backend write error: %d (%s)",
+		    errno, strerror(errno));
+		VBE_CloseFd(sp);
 		/* XXX: other cleanup ? */
-		return (__LINE__);
+		return (retry);
 	}
 
 	/* Checkpoint the shmlog here */
@@ -391,31 +403,39 @@ FetchHdr(struct sess *sp)
 	HTC_Init(sp->wrk->htc, sp->wrk->ws, vc->fd);
 
 	TCP_set_read_timeout(vc->fd, vc->first_byte_timeout);
-	tmo = vc->first_byte_timeout;
 
-	do {
+	i = HTC_Rx(sp->wrk->htc);
+
+	if (i < 0) {
+		WSP(sp, SLT_FetchError, "http first read error: %d %d (%s)",
+		    i, errno, strerror(errno));
+		VBE_CloseFd(sp);
+		/* XXX: other cleanup ? */
+		/* Retryable if we never received anything */
+		return (i == -1 ? retry : -1);
+	}
+
+	TCP_set_read_timeout(vc->fd, vc->between_bytes_timeout);
+
+	while (i == 0) {
 		i = HTC_Rx(sp->wrk->htc);
 		if (i < 0) {
 			WSP(sp, SLT_FetchError,
-			    "http read error: %d", errno);
-			VBE_ClosedFd(sp);
+			    "http first read error: %d %d (%s)",
+			    i, errno, strerror(errno));
+			VBE_CloseFd(sp);
 			/* XXX: other cleanup ? */
-			return (__LINE__);
+			return (-1);
 		}
-
-		if (vc->between_bytes_timeout != tmo) {
-			TCP_set_read_timeout(vc->fd, vc->between_bytes_timeout);
-			tmo = vc->between_bytes_timeout;
-		}
-	} while (i == 0);
+	}
 
 	hp = sp->wrk->beresp;
 
 	if (http_DissectResponse(sp->wrk, sp->wrk->htc, hp)) {
 		WSP(sp, SLT_FetchError, "http format error");
-		VBE_ClosedFd(sp);
+		VBE_CloseFd(sp);
 		/* XXX: other cleanup ? */
-		return (__LINE__);
+		return (-1);
 	}
 	return (0);
 }
@@ -447,7 +467,10 @@ FetchBody(struct sess *sp)
 
 	is_head = (strcasecmp(http_GetReq(sp->wrk->bereq), "head") == 0);
 
-	/* Determine if we have a body or not */
+	/*
+	 * Determine if we have a body or not
+	 * XXX: Missing:  RFC2616 sec. 4.4 in re 1xx, 204 & 304 responses
+	 */
 	cls = 0;
 	mklen = 0;
 	if (is_head) {
@@ -464,9 +487,10 @@ FetchBody(struct sess *sp)
 		sp->wrk->stats.fetch_bad++;
 		/* XXX: AUGH! */
 		WSL(sp->wrk, SLT_Debug, vc->fd, "Invalid Transfer-Encoding");
-		VBE_ClosedFd(sp);
+		VBE_CloseFd(sp);
 		return (__LINE__);
 	} else if (http_HdrIs(hp, H_Connection, "keep-alive")) {
+		sp->wrk->stats.fetch_zero++;
 		/*
 		 * If we have Connection: keep-alive, it cannot possibly be
 		 * EOF encoded, and since it is neither length nor chunked
@@ -489,11 +513,12 @@ FetchBody(struct sess *sp)
 		cls = fetch_eof(sp, sp->wrk->htc);
 		mklen = 1;
 	} else {
-		sp->wrk->stats.fetch_zero++;
+		sp->wrk->stats.fetch_eof++;
 		/*
-		 * Assume zero length
-		 * XXX:  ???
+		 * This is what happens when HTTP/1.0 backends claim
+		 * to be HTTP/1.1, assume EOF
 		 */
+		cls = fetch_eof(sp, sp->wrk->htc);
 		mklen = 1;
 	}
 
@@ -512,7 +537,7 @@ FetchBody(struct sess *sp)
 			VTAILQ_REMOVE(&sp->obj->store, st, list);
 			STV_free(st);
 		}
-		VBE_ClosedFd(sp);
+		VBE_CloseFd(sp);
 		sp->obj->len = 0;
 		return (__LINE__);
 	}
@@ -537,7 +562,7 @@ FetchBody(struct sess *sp)
 		cls = 1;
 
 	if (cls)
-		VBE_ClosedFd(sp);
+		VBE_CloseFd(sp);
 	else
 		VBE_RecycleFd(sp);
 
