@@ -32,7 +32,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_vrt.c 5047 2010-07-13 11:07:00Z tfheen $")
+SVNID("$Id: cache_vrt.c 5361 2010-09-30 06:24:06Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -280,6 +280,7 @@ VRT_l_obj_status(const struct sess *sp, int num)
 	else
 		sprintf(p, "%d", num);
 	http_SetH(sp->obj->http, HTTP_HDR_STATUS, p);
+	sp->obj->http->status = num;
 }
 
 /* Add an objecthead to the saintmode list for the (hopefully) relevant
@@ -360,6 +361,7 @@ VRT_l_resp_status(const struct sess *sp, int num)
 	else
 		sprintf(p, "%d", num);
 	http_SetH(sp->wrk->resp, HTTP_HDR_STATUS, p);
+	sp->wrk->resp->status = num;
 }
 
 int
@@ -367,12 +369,14 @@ VRT_r_resp_status(const struct sess *sp)
 {
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->wrk->resp, HTTP_MAGIC);
+	if (sp->wrk->resp->status)
+		return (sp->wrk->resp->status);
 	return (atoi(sp->wrk->resp->hd[HTTP_HDR_STATUS].b));
 }
 
 /*--------------------------------------------------------------------*/
 
-#define VBEREQ(dir, type,onm,field)					\
+#define VBEREQ(dir, type, onm, field)					\
 void									\
 VRT_l_##dir##_##onm(const struct sess *sp, type a)			\
 {									\
@@ -390,6 +394,30 @@ VRT_r_##dir##_##onm(const struct sess *sp)				\
 VBEREQ(beresp, unsigned, cacheable, cacheable)
 VBEREQ(beresp, double, grace, grace)
 
+/*--------------------------------------------------------------------*/
+
+const char *
+VRT_r_client_identity(struct sess *sp)
+{
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	if (sp->client_identity != NULL)
+		return (sp->client_identity);
+	else
+		return (sp->addr);
+}
+
+void
+VRT_l_client_identity(struct sess *sp, const char *str, ...)
+{
+	va_list ap;
+	char *b;
+
+	va_start(ap, str);
+	b = vrt_assemble_string(sp->http, NULL, str, ap);
+	va_end(ap);
+	sp->client_identity = b;
+}
+
 /*--------------------------------------------------------------------
  * XXX: Working relative to t_req is maybe not the right thing, we could
  * XXX: have spent a long time talking to the backend since then.
@@ -438,6 +466,7 @@ VRT_l_beresp_status(const struct sess *sp, int num)
 	else
 		sprintf(p, "%d", num);
 	http_SetH(sp->wrk->beresp, HTTP_HDR_STATUS, p);
+	sp->wrk->beresp->status = num;
 }
 
 int
@@ -684,6 +713,42 @@ VRT_r_req_xid(struct sess *sp)
 	return (p);
 }
 
+/*--------------------------------------------------------------------
+ * req.hash_ignore_busy
+ */
+
+void
+VRT_l_req_hash_ignore_busy(struct sess *sp, unsigned ignore_busy)
+{
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	sp->hash_ignore_busy = !!ignore_busy;
+}
+
+unsigned
+VRT_r_req_hash_ignore_busy(struct sess *sp)
+{
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	return sp->hash_ignore_busy;
+}
+
+/*--------------------------------------------------------------------
+ * req.hash_always_miss
+ */
+
+void
+VRT_l_req_hash_always_miss(struct sess *sp, unsigned always_miss)
+{
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	sp->hash_always_miss = !!always_miss;
+}
+
+unsigned
+VRT_r_req_hash_always_miss(struct sess *sp)
+{
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	return sp->hash_always_miss;
+}
+
 /*--------------------------------------------------------------------*/
 
 struct sockaddr *
@@ -729,6 +794,10 @@ VRT_r_server_hostname(struct sess *sp)
 	return (vrt_hostname);
 }
 
+/*--------------------------------------------------------------------
+ * XXX: This is pessimistically silly
+ */
+
 int
 VRT_r_server_port(struct sess *sp)
 {
