@@ -33,7 +33,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: hash_critbit.c 5046 2010-07-13 10:26:47Z tfheen $")
+SVNID("$Id: hash_critbit.c 5368 2010-09-30 09:36:13Z tfheen $")
 
 #include <pthread.h>
 #include <stdio.h>
@@ -469,12 +469,10 @@ hcb_lookup(const struct sess *sp, struct objhead *noh)
 		 * under us, so fall through and try with the lock held.
 		 */
 		u = oh->refcnt;
-		if (u > 0) {
+		if (u > 0)
 			oh->refcnt++;
-		} else {
-			assert(!with_lock);
+		else 
 			with_lock = 1;
-		}
 		Lck_Unlock(&oh->mtx);
 		if (u > 0)
 			return (oh);
