@@ -32,7 +32,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: mgt_child.c 4708 2010-04-21 10:36:12Z tfheen $")
+SVNID("$Id: mgt_child.c 5366 2010-09-30 09:04:37Z tfheen $")
 
 #include <unistd.h>
 #include <stdio.h>
@@ -200,14 +200,33 @@ child_poker(const struct vev *e, int what)
 		return (0);
 	if (!mgt_cli_askchild(NULL, NULL, "ping\n"))
 		return (0);
-	REPORT(LOG_ERR,
-	    "Child (%jd) not responding to ping, killing it.",
+	return (0);
+}
+
+/*--------------------------------------------------------------------
+ * If CLI communications with the child process fails, there is nothing
+ * for us to do but to drag it behind the barn and get it over with.
+ *
+ * The typical case is where the child process fails to return a reply
+ * before the cli_timeout expires.  This invalidates the CLI pipes for
+ * all future use, as we don't know if the child was just slow and the
+ * result gets piped later on, or if the child is catatonic.
+ */
+
+void
+MGT_Child_Cli_Fail(void)
+{
+
+	if (child_state != CH_RUNNING)
+		return;
+	if (child_pid < 0)
+		return;
+	REPORT(LOG_ERR, "Child (%jd) not responding to CLI, killing it.",
 	    (intmax_t)child_pid);
 	if (params->diag_bitmap & 0x1000)
 		(void)kill(child_pid, SIGKILL);
 	else
 		(void)kill(child_pid, SIGQUIT);
-	return (0);
 }
 
 /*--------------------------------------------------------------------*/
@@ -267,23 +286,30 @@ waive_privileges(void)
 {
 
 #ifdef HAVE_SETPPRIV
-	priv_set_t *empty;
+	priv_set_t *empty, *minimal;
 
-	if (!(empty = priv_allocset())) {
+	if (!(empty = priv_allocset()) ||
+	    !(minimal = priv_allocset())) {
 		perror("priv_allocset_failed");
 		return;
 	}
 	priv_emptyset(empty);
+	priv_emptyset(minimal);
+
+	/* new privilege, silently ignore any errors if it doesn't exist */
+	priv_addset(minimal, "net_access");
 
 #define SETPPRIV(which, set)				       \
 	if (setppriv(PRIV_SET, which, set))		       \
 		perror("Waiving privileges failed on " #which)
 
-	SETPPRIV(PRIV_LIMIT, empty);
+	/* need to set I after P to avoid SNOCD being set */
+	SETPPRIV(PRIV_LIMIT, minimal);
+	SETPPRIV(PRIV_PERMITTED, minimal); /* implies PRIV_EFFECTIVE */
 	SETPPRIV(PRIV_INHERITABLE, empty);
-	SETPPRIV(PRIV_PERMITTED, empty); /* implies PRIV_EFFECTIVE */
 
 	priv_freeset(empty);
+	priv_freeset(minimal);
 #else
 	return;
 #endif
