@@ -32,7 +32,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: mgt_cli.c 4679 2010-04-19 11:24:21Z tfheen $")
+SVNID("$Id: mgt_cli.c 5332 2010-09-29 08:26:00Z tfheen $")
 
 #include <sys/types.h>
 
@@ -99,6 +99,8 @@ mcf_banner(struct cli *cli, const char *const *av, void *priv)
 	cli_out(cli, "-----------------------------\n");
 	cli_out(cli, "Varnish HTTP accelerator CLI.\n");
 	cli_out(cli, "-----------------------------\n");
+	cli_out(cli, "%s\n", vsb_data(vident) + 1);
+	cli_out(cli, "\n");
 	cli_out(cli, "Type 'help' for command list.\n");
 	cli_out(cli, "Type 'quit' to close CLI session.\n");
 	if (child_pid < 0)
@@ -154,7 +156,6 @@ mcf_askchild(struct cli *cli, const char * const *av, void *priv)
 	char *q;
 	unsigned u;
 
-	(void)av;
 	(void)priv;
 	/*
 	 * Command not recognized in master, try cacher if it is
@@ -177,18 +178,17 @@ mcf_askchild(struct cli *cli, const char * const *av, void *priv)
 	if (i != strlen(cli->cmd)) {
 		cli_result(cli, CLIS_COMMS);
 		cli_out(cli, "CLI communication error");
+		MGT_Child_Cli_Fail();
 		return;
 	}
 	i = write(cli_o, "\n", 1);
 	if (i != 1) {
 		cli_result(cli, CLIS_COMMS);
 		cli_out(cli, "CLI communication error");
+		MGT_Child_Cli_Fail();
 		return;
 	}
-
-	assert(i == 1 || errno == EPIPE);
-	(void)cli_readres(cli_i,
-	    &u, &q, params->cli_timeout);
+	(void)cli_readres(cli_i, &u, &q, params->cli_timeout);
 	cli_result(cli, u);
 	cli_out(cli, "%s", q);
 	free(q);
@@ -233,12 +233,15 @@ mgt_cli_askchild(unsigned *status, char **resp, const char *fmt, ...) {
 			*status = CLIS_COMMS;
 		if (resp != NULL)
 			*resp = strdup("CLI communication error");
+		MGT_Child_Cli_Fail();
 		return (CLIS_COMMS);
 	}
 
 	(void)cli_readres(cli_i, &u, resp, params->cli_timeout);
 	if (status != NULL)
 		*status = u;
+	if (u == CLIS_COMMS)
+		MGT_Child_Cli_Fail();
 	return (u == CLIS_OK ? 0 : u);
 }
 
@@ -440,7 +443,7 @@ sock_id(const char *pfx, int fd)
 	AN(vsb);
 	TCP_myname(fd, abuf1, sizeof abuf1, pbuf1, sizeof pbuf1);
 	TCP_hisname(fd, abuf2, sizeof abuf2, pbuf2, sizeof pbuf2);
-	vsb_printf(vsb, "%s %s:%s %s:%s", pfx, abuf2, pbuf2, abuf1, pbuf1);
+	vsb_printf(vsb, "%s %s %s %s %s", pfx, abuf2, pbuf2, abuf1, pbuf1);
 	vsb_finish(vsb);
 	AZ(vsb_overflowed(vsb));
 	return (vsb);
@@ -532,12 +535,10 @@ void
 mgt_cli_telnet(const char *T_arg)
 {
 	struct vss_addr **ta;
-	char *addr, *port;
 	int i, n, sock, good;
 	struct telnet *tn;
 
-	XXXAZ(VSS_parse(T_arg, &addr, &port));
-	n = VSS_resolve(addr, port, &ta);
+	n = VSS_resolve(T_arg, NULL, &ta);
 	if (n == 0) {
 		fprintf(stderr, "Could not open management port\n");
 		exit(2);
@@ -560,19 +561,15 @@ mgt_cli_telnet(const char *T_arg)
 	}
 	free(ta);
 	if (good == 0) {
-		REPORT(LOG_ERR, "-T %s:%s could not be listened on.",
-		    addr, port);
+		REPORT(LOG_ERR, "-T %s could not be listened on.", T_arg);
 		exit(2);
 	}
-	free(addr);
-	free(port);
 }
 
 /* Reverse CLI ("Master") connections --------------------------------*/
 
 static int M_fd = -1;
 static struct vev *M_poker, *M_conn;
-static char *M_addr, *M_port;
 static struct vss_addr **M_ta;
 static int M_nta, M_nxt;
 static double M_poll = 0.1;
@@ -648,11 +645,7 @@ mgt_cli_master(const char *M_arg)
 {
 	(void)M_arg;
 
-	if (VSS_parse(M_arg, &M_addr, &M_port) || M_port == NULL) {
-		fprintf(stderr, "Could not parse -M argument\n");
-		exit (1);
-	}
-	M_nta = VSS_resolve(M_addr, M_port, &M_ta);
+	M_nta = VSS_resolve(M_arg, NULL, &M_ta);
 	if (M_nta <= 0) {
 		fprintf(stderr, "Could resolve -M argument to address\n");
 		exit (1);
