@@ -32,7 +32,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: mgt_vcc.c 4676 2010-04-19 11:04:19Z tfheen $")
+SVNID("$Id: mgt_vcc.c 5332 2010-09-29 08:26:00Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -327,7 +327,6 @@ mgt_vcc_delbyname(const char *name)
 int
 mgt_vcc_default(const char *b_arg, const char *f_arg, char *vcl, int C_flag)
 {
-	char *addr, *port;
 	char *vf;
 	struct vsb *sb;
 	struct vclprog *vp;
@@ -345,26 +344,10 @@ mgt_vcc_default(const char *b_arg, const char *f_arg, char *vcl, int C_flag)
 		 * XXX: a bug for a backend to not reply at that time, so then
 		 * XXX: again: we should check it here in the "trivial" case.
 		 */
-		if (VSS_parse(b_arg, &addr, &port) != 0 || addr == NULL) {
-			/*
-			 * (addr == NULL && port != NULL) is possible if
-			 * the user incorrectly specified an address such
-			 * as ":80", which is a valid listening address.
-			 * In the future, we may want to interpret this as
-			 * a shortcut for "localhost:80".
-			 */
-			free(port);
-			fprintf(stderr, "invalid backend address\n");
-			return (1);
-		}
-
 		bprintf(buf,
 		    "backend default {\n"
 		    "    .host = \"%s\";\n"
-		    "    .port = \"%s\";\n"
-		    "}\n", addr, port ? port : "http");
-		free(addr);
-		free(port);
+		    "}\n", b_arg);
 		vcl = strdup(buf);
 		AN(vcl);
 	}
@@ -633,6 +616,8 @@ mcf_config_list(struct cli *cli, const char * const *av, void *priv)
 
 /*
  * XXX: This should take an option argument to show all (include) files
+ * XXX: This violates the principle of not loading VCL's in the master
+ * XXX: process.
  */
 void
 mcf_config_show(struct cli *cli, const char * const *av, void *priv)
@@ -654,7 +639,7 @@ mcf_config_show(struct cli *cli, const char * const *av, void *priv)
 			AZ(dlclose(dlh));
 		} else {
 			src = sym;
-			cli_out(cli, src[0]);
+			cli_out(cli, "%s", src[0]);
 			/* cli_out(cli, src[1]); */
 			AZ(dlclose(dlh));
 		}
