@@ -29,7 +29,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vtc_http.c 4761 2010-05-05 07:40:13Z tfheen $")
+SVNID("$Id: vtc_http.c 5362 2010-09-30 06:47:31Z tfheen $")
 
 #include <poll.h>
 #include <stdio.h>
@@ -115,8 +115,8 @@ synth_body(const char *len)
 				k = '!';
 			l = k;
 		} else {
-			b[j] = l++;
-			if (l == '~')
+		  b[j] = (char)l;
+			if (++l == '~')
 				l = '!';
 		}
 	}
@@ -505,9 +505,11 @@ cmd_http_txresp(CMD_ARGS)
 	const char *proto = "HTTP/1.1";
 	const char *status = "200";
 	const char *msg = "Ok";
-	int bodylen;
+	int bodylen = 0;
 	char *b, *c;
-	char *body = NULL;
+	char *body = NULL, *nullbody;
+	int nolen = 0;
+
 
 	(void)cmd;
 	(void)vl;
@@ -518,6 +520,10 @@ cmd_http_txresp(CMD_ARGS)
 
 	vsb_clear(hp->vsb);
 
+	/* send a "Content-Length: 0" header unless something else happens */
+	REPLACE(body, "");
+	nullbody = body;
+
 	for(; *av != NULL; av++) {
 		if (!strcmp(*av, "-proto")) {
 			proto = av[1];
@@ -536,7 +542,9 @@ cmd_http_txresp(CMD_ARGS)
 	vsb_printf(hp->vsb, "%s %s %s%s", proto, status, msg, nl);
 
 	for(; *av != NULL; av++) {
-		if (!strcmp(*av, "-hdr")) {
+		if (!strcmp(*av, "-nolen")) {
+			nolen = 1;
+		} else if (!strcmp(*av, "-hdr")) {
 			vsb_printf(hp->vsb, "%s%s", av[1], nl);
 			av++;
 		} else
@@ -544,8 +552,9 @@ cmd_http_txresp(CMD_ARGS)
 	}
 	for(; *av != NULL; av++) {
 		if (!strcmp(*av, "-body")) {
-			AZ(body);
+			assert(body == nullbody);
 			REPLACE(body, av[1]);
+
 			AN(body);
 			av++;
 			bodylen = strlen(body);
@@ -560,7 +569,7 @@ cmd_http_txresp(CMD_ARGS)
 				}
 			}
 		} else if (!strcmp(*av, "-bodylen")) {
-			AZ(body);
+			assert(body == nullbody);
 			body = synth_body(av[1]);
 			bodylen = strlen(body);
 			av++;
@@ -569,7 +578,7 @@ cmd_http_txresp(CMD_ARGS)
 	}
 	if (*av != NULL)
 		vtc_log(hp->vl, 0, "Unknown http txresp spec: %s\n", *av);
-	if (body != NULL)
+	if (body != NULL && !nolen)
 		vsb_printf(hp->vsb, "Content-Length: %d%s", bodylen, nl);
 	vsb_cat(hp->vsb, nl);
 	if (body != NULL)
