@@ -29,7 +29,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vtc_server.c 5036 2010-07-13 08:16:48Z tfheen $")
+SVNID("$Id: vtc_server.c 5329 2010-09-29 08:03:08Z tfheen $")
 
 #include <stdio.h>
 #include <errno.h>
@@ -94,7 +94,7 @@ server_thread(void *priv)
 
 	vl = vtc_logopen(s->name);
 
-	vtc_log(vl, 2, "Started on %s:%s", s->aaddr, s->aport);
+	vtc_log(vl, 2, "Started on %s %s", s->aaddr, s->aport);
 	for (i = 0; i < s->repeat; i++) {
 		if (s->repeat > 1)
 			vtc_log(vl, 3, "Iteration %d", i);
@@ -110,9 +110,6 @@ server_thread(void *priv)
 			vtc_log(vl, 0, "Shutdown failed: %s", strerror(errno));
 		TCP_close(&fd);
 	}
-	macro_def(s->vl, s->name, "addr", NULL);
-	macro_def(s->vl, s->name, "port", NULL);
-	macro_def(s->vl, s->name, "sock", NULL);
 	vtc_log(vl, 2, "Ending");
 	return (NULL);
 }
@@ -153,6 +150,9 @@ server_delete(struct server *s)
 {
 
 	CHECK_OBJ_NOTNULL(s, SERVER_MAGIC);
+	macro_def(s->vl, s->name, "addr", NULL);
+	macro_def(s->vl, s->name, "port", NULL);
+	macro_def(s->vl, s->name, "sock", NULL);
 	vtc_logclose(s->vl);
 	free(s->listen);
 	free(s->name);
@@ -184,12 +184,12 @@ server_start(struct server *s)
 		    s->aport, sizeof s->aport);
 		macro_def(s->vl, s->name, "addr", "%s", s->aaddr);
 		macro_def(s->vl, s->name, "port", "%s", s->aport);
-		macro_def(s->vl, s->name, "sock", "%s:%s", s->aaddr, s->aport);
+		macro_def(s->vl, s->name, "sock", "%s %s", s->aaddr, s->aport);
 		/* Record the actual port, and reuse it on subsequent starts */
 		if (!strcmp(s->port, "0"))
 			REPLACE(s->port, s->aport);
 	}
-	vtc_log(s->vl, 1, "Listen on %s:%s", s->addr, s->port);
+	vtc_log(s->vl, 1, "Listen on %s %s", s->addr, s->port);
 	s->run = 1;
 	AZ(pthread_create(&s->tp, NULL, server_thread, s));
 }
