@@ -29,7 +29,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vtc_varnish.c 4770 2010-05-05 08:38:16Z tfheen $")
+SVNID("$Id: vtc_varnish.c 5329 2010-09-29 08:03:08Z tfheen $")
 
 #include <stdio.h>
 
@@ -198,7 +198,8 @@ varnish_delete(struct varnish *v)
 	vtc_logclose(v->vl);
 	free(v->name);
 	free(v->workdir);
-	VSL_Close();
+	if (v->stats != NULL)
+		VSL_Close();
 
 	/*
 	 * We do not delete the workdir, it may contain stuff people
@@ -276,7 +277,7 @@ varnish_launch(struct varnish *v)
 	vsb_printf(vsb, " -p syslog_cli_traffic=off");
 	vsb_printf(vsb, " -a '%s'", "127.0.0.1:0");
 	vsb_printf(vsb, " -S %s/_S", v->workdir);
-	vsb_printf(vsb, " -M %s:%s", abuf, pbuf);
+	vsb_printf(vsb, " -M '%s %s'", abuf, pbuf);
 	vsb_printf(vsb, " -P %s/varnishd.pid", v->workdir);
 	vsb_printf(vsb, " %s", vsb_data(v->storage));
 	vsb_printf(vsb, " %s", vsb_data(v->args));
@@ -407,7 +408,7 @@ varnish_start(struct varnish *v)
 	vtc_log(v->vl, 2, "Listen on %s %s", h, p);
 	macro_def(v->vl, v->name, "addr", "%s", h);
 	macro_def(v->vl, v->name, "port", "%s", p);
-	macro_def(v->vl, v->name, "sock", "%s:%s", h, p);
+	macro_def(v->vl, v->name, "sock", "%s %s", h, p);
 }
 
 /**********************************************************************
