@@ -33,7 +33,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: varnishtop.c 4530 2010-02-03 08:40:23Z tfheen $")
+SVNID("$Id: varnishtop.c 5150 2010-08-30 12:01:05Z tfheen $")
 
 #include <ctype.h>
 #include <curses.h>
@@ -55,7 +55,7 @@ SVNID("$Id: varnishtop.c 4530 2010-02-03 08:40:23Z tfheen $")
 #include "varnishapi.h"
 
 struct top {
-	unsigned char		rec[4];
+	unsigned char		rec[SHMLOG_DATA-1];
 	unsigned char		*rec_data;
 	unsigned		clen;
 	unsigned		hash;
