@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vrt.h 5047 2010-07-13 11:07:00Z tfheen $
+ * $Id: vrt.h 5434 2010-10-19 15:01:04Z tfheen $
  *
  * Runtime support for compiled VCL programs.
  *
@@ -68,7 +68,10 @@ struct vrt_backend {
 	const char			*hosthdr;
 
 	const unsigned char		*ipv4_sockaddr;
+	const char			*ipv4_addr;
 	const unsigned char		*ipv6_sockaddr;
+	const char			*ipv6_addr;
+	const char			*port;
 
 	double				connect_timeout;
 	double				first_byte_timeout;
@@ -108,6 +111,21 @@ struct vrt_dir_round_robin {
 	const struct vrt_dir_round_robin_entry	*members;
 };
 
+/*
+ * A director with dns-based selection
+ */
+
+struct vrt_dir_dns_entry {
+	int					host;
+};
+
+struct vrt_dir_dns {
+	const char				*name;
+	const char				*suffix;
+	const double				ttl;
+	unsigned				nmember;
+	const struct vrt_dir_dns_entry		*members;
+};
 
 /*
  * other stuff.
