@@ -30,6 +30,8 @@
 #include "libvarnish.h"
 #include "vmb.h"
 
+#ifdef VMB_NEEDS_PTHREAD_WORKAROUND_THIS_IS_BAD_FOR_PERFORMANCE
+
 static pthread_mutex_t	mb_mtx;
 static pthread_once_t	mb_mtx_once = PTHREAD_ONCE_INIT;
 
@@ -50,3 +52,5 @@ vmb_pthread(void)
 	AZ(pthread_mutex_lock(&mb_mtx));
 	AZ(pthread_mutex_unlock(&mb_mtx));
 }
+
+#endif
