@@ -31,7 +31,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vss.c 4588 2010-02-26 09:35:42Z phk $")
+SVNID("$Id: vss.c 5333 2010-09-29 08:31:36Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -79,7 +79,10 @@ struct vss_addr {
  * "0.0.0.0" - "0.0.0.0:80"
  * "[::1]" - "[::1]:80"
  * "[::]" - "[::]:80"
+ *
+ * See also RFC5952
  */
+
 int
 VSS_parse(const char *str, char **addr, char **port)
 {
@@ -101,7 +104,9 @@ VSS_parse(const char *str, char **addr, char **port)
 		}
 	} else {
 		/* IPv4 address of the form 127.0.0.1:80, or non-numeric */
-		p = strchr(str, ':');
+		p = strchr(str, ' ');
+		if (p == NULL)
+			p = strchr(str, ':');
 		if (p == NULL) {
 			*addr = strdup(str);
 			XXXAN(*addr);
@@ -129,6 +134,9 @@ VSS_parse(const char *str, char **addr, char **port)
  *
  * The return value is the number of addresses resoved, or zero.
  *
+ * If the addr argument contains a port specification, that takes
+ * precedence over the port argument.
+ *
  * XXX: We need a function to free the allocated addresses.
  */
 int
@@ -137,15 +145,27 @@ VSS_resolve(const char *addr, const char *port, struct vss_addr ***vap)
 	struct addrinfo hints, *res0, *res;
 	struct vss_addr **va;
 	int i, ret;
+	char *adp, *hop;
 
 	memset(&hints, 0, sizeof hints);
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_flags = AI_PASSIVE;
-	ret = getaddrinfo(addr, port, &hints, &res0);
-	if (ret != 0) {
-		fprintf(stderr, "getaddrinfo(): %s\n", gai_strerror(ret));
+
+	ret = VSS_parse(addr, &hop, &adp);
+	if (ret)
 		return (0);
-	}
+
+	if (adp == NULL)
+		ret = getaddrinfo(addr, port, &hints, &res0);
+	else
+		ret = getaddrinfo(hop, adp, &hints, &res0);
+
+	free(hop);
+	free(adp);
+
+	if (ret != 0)
+		return (0);
+
 	XXXAN(res0);
 	for (res = res0, i = 0; res != NULL; res = res->ai_next, ++i)
 		/* nothing */ ;
