@@ -51,7 +51,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vcc_backend.c 5041 2010-07-13 09:10:48Z tfheen $")
+SVNID("$Id: vcc_backend.c 5434 2010-10-19 15:01:04Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -60,9 +60,11 @@ SVNID("$Id: vcc_backend.c 5041 2010-07-13 09:10:48Z tfheen $")
 #include <netdb.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "vsb.h"
+#include "vss.h"
 
 #include "vcc_priv.h"
 #include "vcc_compile.h"
@@ -74,20 +76,30 @@ struct host {
 	char			*vgcname;
 };
 
-static const char *
-CheckHostPort(const char *host, const char *port)
+static int
+emit_sockaddr(struct tokenlist *tl, void *sa, unsigned sal)
 {
-	struct addrinfo *res, hint;
-	int error;
+	unsigned len;
+	uint8_t *u;
 
-	memset(&hint, 0, sizeof hint);
-	hint.ai_family = PF_UNSPEC;
-	hint.ai_socktype = SOCK_STREAM;
-	error = getaddrinfo(host, port, &hint, &res);
-	if (error)
-		return (gai_strerror(error));
-	freeaddrinfo(res);
-	return (NULL);
+	AN(sa);
+	AN(sal);
+	assert(sal < 256);
+	Fh(tl, 0, "\nstatic const unsigned char sockaddr%u[%d] = {\n",
+	    tl->nsockaddr, sal + 1);
+	Fh(tl, 0, "    %3u, /* Length */\n",  sal);
+	u = sa;
+	for (len = 0; len <sal; len++) {
+		if ((len % 8) == 0)
+			Fh(tl, 0, "   ");
+		Fh(tl, 0, " %3u", u[len]);
+		if (len + 1 < sal)
+			Fh(tl, 0, ",");
+		if ((len % 8) == 7)
+			Fh(tl, 0, "\n");
+	}
+	Fh(tl, 0, "\n};\n");
+	return (tl->nsockaddr++);
 }
 
 /*--------------------------------------------------------------------
@@ -96,35 +108,63 @@ CheckHostPort(const char *host, const char *port)
  * and put it in an official sockaddr when we load the VCL.
  */
 
-static void
+#include <stdio.h>
+
+void
 Emit_Sockaddr(struct tokenlist *tl, const struct token *t_host,
     const char *port)
+
 {
 	struct addrinfo *res, *res0, *res1, hint;
-	int n4, n6, len, error, retval;
+	int n4, n6, error, retval, x;
 	const char *emit, *multiple;
-	unsigned char *u;
 	char hbuf[NI_MAXHOST];
+	char *hop, *pop;
 
 	AN(t_host->dec);
 	retval = 0;
 	memset(&hint, 0, sizeof hint);
 	hint.ai_family = PF_UNSPEC;
 	hint.ai_socktype = SOCK_STREAM;
-	error = getaddrinfo(t_host->dec, port, &hint, &res0);
+
+	if (VSS_parse(t_host->dec, &hop, &pop)) {
+		vsb_printf(tl->sb,
+		    "Backend host '%.*s': wrong syntax (unbalanced [...] ?)\n",
+		    PF(t_host) );
+		vcc_ErrWhere(tl, t_host);
+		return;
+	}
+	if (pop != NULL)
+		error = getaddrinfo(hop, pop, &hint, &res0);
+	else
+		error = getaddrinfo(t_host->dec, port, &hint, &res0);
+	free(hop);
+	free(pop);
+	if (error) {
+		vsb_printf(tl->sb,
+		    "Backend host '%.*s'"
+		    " could not be resolved to an IP address:\n", PF(t_host));
+		vsb_printf(tl->sb,
+		    "\t%s\n"
+		    "(Sorry if that error message is gibberish.)\n",
+		    gai_strerror(error));
+		vcc_ErrWhere(tl, t_host);
+		return;
+	}
 	AZ(error);
 	n4 = n6 = 0;
 	multiple = NULL;
+
 	for (res = res0; res; res = res->ai_next) {
 		emit = NULL;
 		if (res->ai_family == PF_INET) {
 			if (n4++ == 0)
-				emit = "ipv4_sockaddr";
+				emit = "ipv4";
 			else
 				multiple = "IPv4";
 		} else if (res->ai_family == PF_INET6) {
 			if (n6++ == 0)
-				emit = "ipv6_sockaddr";
+				emit = "ipv6";
 			else
 				multiple = "IPv6";
 		} else
@@ -149,26 +189,22 @@ Emit_Sockaddr(struct tokenlist *tl, const struct token *t_host,
 			return;
 		}
 		AN(emit);
-		AN(res->ai_addr);
-		AN(res->ai_addrlen);
-		assert(res->ai_addrlen < 256);
-		Fh(tl, 0, "\nstatic const unsigned char sockaddr%u[%d] = {\n",
-		    tl->nsockaddr, res->ai_addrlen + 1);
-		Fh(tl, 0, "    %3u, /* Length */\n",  res->ai_addrlen);
-		u = (void*)res->ai_addr;
-		for (len = 0; len < res->ai_addrlen; len++) {
-			if ((len % 8) == 0)
-				Fh(tl, 0, "   ");
-			Fh(tl, 0, " %3u", u[len]);
-			if (len + 1 < res->ai_addrlen)
-				Fh(tl, 0, ",");
-			if ((len % 8) == 7)
-				Fh(tl, 0, "\n");
-		}
-		Fh(tl, 0, "\n};\n");
-		Fb(tl, 0, "\t.%s = sockaddr%u,\n", emit, tl->nsockaddr++);
+		x = emit_sockaddr(tl, res->ai_addr, res->ai_addrlen);
+		Fb(tl, 0, "\t.%s_sockaddr = sockaddr%u,\n", emit, x);
+		error = getnameinfo(res->ai_addr,
+		    res->ai_addrlen, hbuf, sizeof hbuf,
+		    NULL, 0, NI_NUMERICHOST);
+		AZ(error);
+		Fb(tl, 0, "\t.%s_addr = \"%s\",\n", emit, hbuf);
 		retval++;
 	}
+	if (res0 != NULL) {
+		error = getnameinfo(res0->ai_addr,
+		    res0->ai_addrlen, NULL, 0, hbuf, sizeof hbuf,
+		    NI_NUMERICSERV);
+		AZ(error);
+		Fb(tl, 0, "\t.port = \"%s\",\n", hbuf);
+	}
 	freeaddrinfo(res0);
 	if (retval == 0) {
 		vsb_printf(tl->sb,
@@ -189,7 +225,7 @@ Emit_Sockaddr(struct tokenlist *tl, const struct token *t_host,
  * in that context.
  */
 
-static void
+void
 vcc_EmitBeIdent(const struct tokenlist *tl, struct vsb *v,
     int serial, const struct token *first, const struct token *last)
 {
@@ -389,7 +425,6 @@ vcc_ParseHostDef(struct tokenlist *tl, int serial, const char *vgcname)
 	struct token *t_port = NULL;
 	struct token *t_hosthdr = NULL;
 	unsigned saint = UINT_MAX;
-	const char *ep;
 	struct fld_spec *fs;
 	struct vsb *vsb;
 	unsigned u;
@@ -511,26 +546,10 @@ vcc_ParseHostDef(struct tokenlist *tl, int serial, const char *vgcname)
 
 	/* Check that the hostname makes sense */
 	assert(t_host != NULL);
-	ep = CheckHostPort(t_host->dec, "80");
-	if (ep != NULL) {
-		vsb_printf(tl->sb, "Backend host '%.*s': %s\n", PF(t_host), ep);
-		vcc_ErrWhere(tl, t_host);
-		return;
-	}
-
-	/* Check that the portname makes sense */
-	if (t_port != NULL) {
-		ep = CheckHostPort("127.0.0.1", t_port->dec);
-		if (ep != NULL) {
-			vsb_printf(tl->sb,
-			    "Backend port '%.*s': %s\n", PF(t_port), ep);
-			vcc_ErrWhere(tl, t_port);
-			return;
-		}
+	if (t_port != NULL) 
 		Emit_Sockaddr(tl, t_host, t_port->dec);
-	} else {
+	else
 		Emit_Sockaddr(tl, t_host, "80");
-	}
 	ERRCHK(tl);
 
 	ExpectErr(tl, '}');
@@ -662,6 +681,7 @@ static const struct dirlist {
 	{ "random",		vcc_ParseRandomDirector },
 	{ "client",		vcc_ParseRandomDirector },
 	{ "round-robin",	vcc_ParseRoundRobinDirector },
+	{ "dns",		vcc_ParseDnsDirector },
 	{ NULL,		NULL }
 };
 
