@@ -30,10 +30,11 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vcc_parse.c 4693 2010-04-19 12:40:31Z tfheen $")
+SVNID("$Id: vcc_parse.c 5039 2010-07-13 08:52:54Z tfheen $")
 
 #include <stdio.h>
 #include <string.h>
+#include <math.h>
 
 #include "vsb.h"
 
@@ -153,6 +154,8 @@ vcc_DoubleVal(struct tokenlist *tl)
 	const char *p;
 
 	Expect(tl, CNUM);
+	if (tl->err)
+		return (NAN);
 	for (p = tl->t->b; p < tl->t->e; p++) {
 		d *= 10;
 		d += *p - '0';
