@@ -30,7 +30,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vcc_string.c 4428 2010-01-06 17:38:59Z tfheen $")
+SVNID("$Id: vcc_string.c 5372 2010-10-01 07:21:18Z tfheen $")
 
 #include <stdio.h>
 #include <string.h>
@@ -153,6 +153,11 @@ vcc_StringVal(struct tokenlist *tl)
 		return (vcc_regsub(tl, 0));
 	if (tl->t->tok == ID && vcc_IdIs(tl->t, "regsuball"))
 		return (vcc_regsub(tl, 1));
+	if (tl->t->tok == ID && vcc_IdIs(tl->t, "now")) {
+		Fb(tl, 0, "VRT_time_string(sp, VRT_r_now(sp))");
+		vcc_NextToken(tl);
+		return 1;
+	}
 	if (tl->t->tok == VAR) {
 		vp = vcc_FindVar(tl, tl->t, vcc_vars);
 		if (tl->err)
